@@ -42,9 +42,10 @@
  *    fortune with any other parameters, or none, and which forced
  *    the program to read only one file (named 'fortunes')
  * 3: removed the unnecessary print_file_list()
- * 4: Added "OFFDIR" to pathnames.h as the directory in which offensive
- *    fortunes are kept.  This considerably simplifies our life by
- *    permitting us to dispense with a lot of silly tests for the string
+ * 4: Added "FORTUNE_SYSTEM_OFFENSIVE_FORTUNES_DIR" to pathnames.h as the
+ * directory in which offensive fortunes are kept.  This considerably simplifies
+ * our life by permitting us to dispense with a lot of silly tests for the
+ * string
  *    "-o" at the end of a filename.
  * 5: I think the problems with trying to find filenames were fixed by
  *    the change in the way that offensive files are defined.  Two birds,
@@ -114,8 +115,8 @@
 #define MINW 6   /* minimum wait if desired */
 #define CPERS 20 /* # of chars for each sec */
 
-#define POS_UNKNOWN ((int32_t)-1) /* pos for file unknown */
-#define NO_PROB (-1)              /* no prob specified for file */
+#define POS_UNKNOWN ((int32_t)(-1)) /* pos for file unknown */
+#define NO_PROB (-1)                /* no prob specified for file */
 
 #ifdef DEBUG
 #define DPRINTF(l, x)                                                          \
@@ -428,7 +429,9 @@ static FILEDESC *new_fp(void)
     fp->tbl.stuff[1] = 0;
     fp->tbl.stuff[2] = 0;
     fp->tbl.stuff[3] = 0;
+    fp->name = NULL;
     fp->next = NULL;
+    fp->path = NULL;
     fp->prev = NULL;
     fp->child = NULL;
     fp->parent = NULL;
@@ -644,13 +647,17 @@ static int add_file(int percent, const char *file, const char *dir,
                     {
                         *p++ = '\0';
                     }
-                    snprintf(langdir, sizeof(langdir), "%s/%s", FORTDIR, lang);
+                    snprintf(langdir, sizeof(langdir), "%s/%s",
+                        FORTUNE_SYSTEM_INOFFENSIVE_FORTUNES_DIR, lang);
 
                     if (strncmp(path, lang, 2) == 0)
                     {
                         ret = 1;
                     }
-                    else if (strncmp(path, langdir, strlen(FORTDIR) + 3) == 0)
+                    else if (strncmp(path, langdir,
+                                 strlen(
+                                     FORTUNE_SYSTEM_INOFFENSIVE_FORTUNES_DIR) +
+                                     3) == 0)
                     {
                         ret = 1;
                     }
@@ -834,8 +841,8 @@ static int add_dir(FILEDESC *const fp)
          * allowed to be empty.
          *  - Brian Bassett (brianb@debian.org) 1999/07/31
          */
-        if (strcmp(LOCFORTDIR, fp->path) == 0 ||
-            strcmp(LOCOFFDIR, fp->path) == 0)
+        if (strcmp(FORTUNE_LOCAL_INOFFENSIVE_FORTUNES_DIR, fp->path) == 0 ||
+            strcmp(FORTUNE_LOCAL_OFFENSIVE_FORTUNES_DIR, fp->path) == 0)
         {
             return true;
         }
@@ -866,19 +873,22 @@ static int cond_top_level__add_file(
     return top_level__add_file(dirpath);
 }
 
-static int cond_top_level__LOCFORTDIR(void)
+static int cond_top_level__FORTUNEMOD_LOCAL_INOFFENSIVE_FORTUNES_DIR(void)
 {
-    return cond_top_level__add_file(FORTDIR, LOCFORTDIR);
+    return cond_top_level__add_file(FORTUNE_SYSTEM_INOFFENSIVE_FORTUNES_DIR,
+        FORTUNE_LOCAL_INOFFENSIVE_FORTUNES_DIR);
 }
 
-static int cond_top_level__OFFDIR(void)
+static int cond_top_level__FORTUNE_SYSTEM_OFFENSIVE_FORTUNES_DIR(void)
 {
-    return cond_top_level__add_file(OFFDIR, LOCOFFDIR);
+    return cond_top_level__add_file(FORTUNE_SYSTEM_OFFENSIVE_FORTUNES_DIR,
+        FORTUNE_LOCAL_OFFENSIVE_FORTUNES_DIR);
 }
 
-static int top_level_LOCFORTDIR(void)
+static int top_level_FORTUNEMOD_LOCAL_INOFFENSIVE_FORTUNES_DIR(void)
 {
-    return (top_level__add_file(LOCFORTDIR) | cond_top_level__LOCFORTDIR());
+    return (top_level__add_file(FORTUNE_LOCAL_INOFFENSIVE_FORTUNES_DIR) |
+            cond_top_level__FORTUNEMOD_LOCAL_INOFFENSIVE_FORTUNES_DIR());
 }
 
 static int form_file_list(char **files, int file_cnt)
@@ -892,13 +902,16 @@ static int form_file_list(char **files, int file_cnt)
     {
         if (All_forts)
         {
-            return (top_level__add_file(LOCFORTDIR) |
-                    top_level__add_file(LOCOFFDIR) |
-                    cond_top_level__LOCFORTDIR() | cond_top_level__OFFDIR());
+            return (
+                top_level__add_file(FORTUNE_LOCAL_INOFFENSIVE_FORTUNES_DIR) |
+                top_level__add_file(FORTUNE_LOCAL_OFFENSIVE_FORTUNES_DIR) |
+                cond_top_level__FORTUNEMOD_LOCAL_INOFFENSIVE_FORTUNES_DIR() |
+                cond_top_level__FORTUNE_SYSTEM_OFFENSIVE_FORTUNES_DIR());
         }
         else if (Offend)
         {
-            return (top_level__add_file(LOCOFFDIR) | cond_top_level__OFFDIR());
+            return (top_level__add_file(FORTUNE_LOCAL_OFFENSIVE_FORTUNES_DIR) |
+                    cond_top_level__FORTUNE_SYSTEM_OFFENSIVE_FORTUNES_DIR());
         }
         else
         {
@@ -945,12 +958,12 @@ static int form_file_list(char **files, int file_cnt)
                     lang = p;
                 }
                 /* default */
-                return top_level_LOCFORTDIR();
+                return top_level_FORTUNEMOD_LOCAL_INOFFENSIVE_FORTUNES_DIR();
             }
             else
             {
                 /* no locales available, use default */
-                return top_level_LOCFORTDIR();
+                return top_level_FORTUNEMOD_LOCAL_INOFFENSIVE_FORTUNES_DIR();
             }
         }
     }
@@ -958,6 +971,7 @@ static int form_file_list(char **files, int file_cnt)
     for (i = 0; i < file_cnt; i++)
     {
         percent = NO_PROB;
+        sp = NULL;
         if (!isdigit(files[i][0]))
         {
             sp = files[i];
@@ -1030,8 +1044,11 @@ static int form_file_list(char **files, int file_cnt)
             offensive = true;
         }
 
-        const char *fulldir = offensive ? OFFDIR : FORTDIR;
-        const char *locdir = offensive ? LOCOFFDIR : LOCFORTDIR;
+        const char *fulldir = offensive
+                                  ? FORTUNE_SYSTEM_OFFENSIVE_FORTUNES_DIR
+                                  : FORTUNE_SYSTEM_INOFFENSIVE_FORTUNES_DIR;
+        const char *locdir = offensive ? FORTUNE_LOCAL_OFFENSIVE_FORTUNES_DIR
+                                       : FORTUNE_LOCAL_INOFFENSIVE_FORTUNES_DIR;
 
         if (strcmp(sp, "all") == 0)
         {
@@ -1039,7 +1056,8 @@ static int form_file_list(char **files, int file_cnt)
             snprintf(locpathname, sizeof(locpathname), "%s", locdir);
         }
         /* if it isn't an absolute path or relative to . or ..
-           make it an absolute path relative to FORTDIR */
+           make it an absolute path relative to
+           FORTUNE_SYSTEM_INOFFENSIVE_FORTUNES_DIR */
         else
         {
             if (strncmp(sp, "/", 1) != 0 && strncmp(sp, "./", 2) != 0 &&
@@ -1096,22 +1114,66 @@ static int form_file_list(char **files, int file_cnt)
                 lang = p;
             }
             /* default */
+            FILEDESC *system_dir_list = NULL, *local_list = NULL;
+            FILEDESC *system_dir_tail = NULL, *local_tail = NULL;
             if (!ret)
             {
-                ret = add_file(
-                    percent, fullpathname, NULL, &File_list, &File_tail, NULL);
+                ret = add_file(100, fullpathname, NULL, &system_dir_list,
+                    &system_dir_tail, NULL);
             }
-            if (!ret &&
-                strncmp(fullpathname, locpathname, sizeof(fullpathname)))
-            {
-                ret = add_file(
-                    percent, locpathname, NULL, &File_list, &File_tail, NULL);
-            }
-            if (strncmp(fullpathname, locpathname, sizeof(fullpathname)) &&
-                strcmp(sp, "all") == 0)
+            if (strncmp(fullpathname, locpathname, sizeof(fullpathname)))
             {
                 add_file(
-                    percent, locpathname, NULL, &File_list, &File_tail, NULL);
+                    100, locpathname, NULL, &local_list, &local_tail, NULL);
+            }
+            if (!local_list)
+            {
+                if (system_dir_tail)
+                {
+                    system_dir_tail->percent = percent;
+                    if (File_tail)
+                    {
+                        FILEDESC *old = File_tail;
+                        system_dir_list->prev = File_list;
+                        old->next = system_dir_list;
+                        File_tail = system_dir_tail;
+                    }
+                    else
+                    {
+                        File_tail = system_dir_tail;
+                        File_list = system_dir_list;
+                    }
+                }
+                else
+                {
+                    ret = 0;
+                }
+            }
+            else if (strncmp(fullpathname, locpathname, sizeof(fullpathname)) &&
+                     strcmp(sp, "all") == 0)
+            {
+                if (system_dir_list)
+                {
+                    FILEDESC *parent_node = new_fp();
+                    parent_node->percent = percent;
+                    parent_node->child = system_dir_list;
+                    system_dir_list->next = local_list;
+                    if (local_list)
+                    {
+                        local_list->prev = system_dir_list;
+                    }
+                    system_dir_list->parent = parent_node;
+                    if (local_list)
+                    {
+                        local_list->parent = parent_node;
+                    }
+                    File_tail = File_list = parent_node;
+                    ret = 1;
+                }
+                else
+                {
+                    ret = 0;
+                }
             }
             if (!ret)
             {
@@ -1517,7 +1579,6 @@ static FILEDESC *pick_child(FILEDESC *parent)
             continue;
         }
         DPRINTF(1, (stderr, "    using %s\n", fp->name));
-        return fp;
     }
     else
     {
@@ -1526,7 +1587,7 @@ static FILEDESC *pick_child(FILEDESC *parent)
         DPRINTF(1, (stderr, "    choice = %d (of %ld)\n", choice,
                        parent->tbl.str_numstr));
         for (fp = parent->child; choice >= (int)fp->tbl.str_numstr;
-             fp = fp->next)
+            fp = fp->next)
         {
             choice -= fp->tbl.str_numstr;
             DPRINTF(1, (stderr, "\tskip %s, %ld (choice = %d)\n", fp->name,
@@ -1534,8 +1595,8 @@ static FILEDESC *pick_child(FILEDESC *parent)
         }
         DPRINTF(
             1, (stderr, "    using %s, %ld\n", fp->name, fp->tbl.str_numstr));
-        return fp;
     }
+    return (fp->child ? (pick_child(fp)) : fp);
 }
 
 /*
@@ -1636,7 +1697,9 @@ static void get_fort(void)
     {
         DPRINTF(1, (stderr, "%s", "picking child\n"));
         fp = pick_child(fp);
+        assert(!fp->child);
     }
+    assert(!fp->child);
     Fortfile = fp;
     get_pos(fp);
     open_dat(fp);
@@ -1814,7 +1877,7 @@ static void display(FILEDESC *fp)
     }
     for (Fort_len = 0; fgets((char *)line, sizeof line, fp->inf) &&
                        !STR_ENDSTRING(line, fp->tbl);
-         Fort_len++)
+        Fort_len++)
     {
         if (fp->tbl.str_flags & STR_ROTATED)
         {
