@@ -3,8 +3,19 @@
 #include <stdio.h>
 #include <ctype.h>
 
+#ifdef _WIN32
+#include <fcntl.h>
+#include <io.h>
+#endif
+
 int main(void)
 {
+#ifdef _WIN32
+    // Force binary mode to avoid CRLF translation.
+    setmode(_fileno(stdin), _O_BINARY);
+    setmode(_fileno(stdout), _O_BINARY);
+#endif
+
     int a;
     while ((a = getchar()) != EOF)
     {
